@@ -6,18 +6,14 @@ import sys
 import time
 import glob
 import http.cookiejar
+import json
 import tempfile
 import lz4.block
-import datetime
 import configparser
 import base64
 from Crypto.Cipher import AES
 from typing import Union
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
 try:
     # should use pysqlite2 to read the cookies.sqlite on Windows
     # otherwise will raise the "sqlite3.DatabaseError: file is encrypted or is not a database" exception
@@ -108,8 +104,7 @@ def crypt_unprotect_data(
 
 
 def get_linux_pass(os_crypt_name):
-    '''Retrive password used to encrypt cookies from libsecret.
-    '''
+    """Retrieve password used to encrypt cookies from libsecret"""
     # https://github.com/n8henrie/pycookiecheat/issues/12
     my_pass = None
 
@@ -117,14 +112,14 @@ def get_linux_pass(os_crypt_name):
     connection = secretstorage.dbus_init()
     collection = secretstorage.get_default_collection(connection)
     secret = None
-    
+
     # we should not look for secret with label. Sometimes label can be different. For example,
     # if Steam is installed before Chromium, Opera or Edge, it will show Steam Secret Storage as label.
     # insted we should look with schema and application
     secret = next(collection.search_items(
         {'xdg:schema': 'chrome_libsecret_os_crypt_password_v2',
             'application': os_crypt_name}), None)
-    
+
     if not secret:
         # trying os_crypt_v1
         secret = next(collection.search_items(
@@ -145,11 +140,13 @@ def get_linux_pass(os_crypt_name):
             ).encode('utf-8')
         except RuntimeError:
             pass
+        except AttributeError:
+            pass
 
     # try default peanuts password, probably won't work
     if not my_pass:
         my_pass = 'peanuts'.encode('utf-8')
-    
+
     return my_pass
 
 def __expand_win_path(path:Union[dict,str]):
@@ -158,27 +155,33 @@ def __expand_win_path(path:Union[dict,str]):
     return os.path.join(os.getenv(path['env'], ''), path['path'])
 
 def expand_paths(paths:list, os_name:str):
-    '''Expands user paths on Linux, OSX, and windows
-    '''
+    """Expands user paths on Linux, OSX, and windows"""
 
     os_name = os_name.lower()
     assert os_name in ['windows', 'osx', 'linux']
 
     if not isinstance(paths, list):
         paths = [paths]
-    
+
     if os_name == 'windows':
         paths = map(__expand_win_path, paths)
     else:
         paths = map(os.path.expanduser, paths)
-    
+
     paths = next(filter(os.path.exists, paths), None)
     return paths
 
+def text_factory(data):
+    try:
+        return data.decode('utf-8')
+    except UnicodeDecodeError:
+        return data
 
 class ChromiumBased:
-    '''Super class for all Chromium based browser.
-    '''
+    """Super class for all Chromium based browsers"""
+
+    UNIX_TO_NT_EPOCH_OFFSET = 11644473600  # seconds from 1601-01-01T00:00:00Z to 1970-01-01T00:00:00Z
+
     def __init__(self, browser:str, cookie_file=None, domain_name="", key_file=None, **kwargs):
         self.salt = b'saltysalt'
         self.iv = b' ' * 16
@@ -188,13 +191,13 @@ class ChromiumBased:
         self.domain_name = domain_name
         self.key_file = key_file
         self.__add_key_and_cookie_file(**kwargs)
-    
-    def __add_key_and_cookie_file(self, 
+
+    def __add_key_and_cookie_file(self,
             linux_cookies=None, windows_cookies=None, osx_cookies=None,
             windows_keys=None, os_crypt_name=None, osx_key_service=None, osx_key_user=None):
 
         if sys.platform == 'darwin':
-            # running Chromium or it's derivatives on OSX
+            # running Chromium or its derivatives on OSX
             my_pass = keyring.get_password(osx_key_service, osx_key_user)
 
             # try default peanuts password, probably won't work
@@ -205,19 +208,18 @@ class ChromiumBased:
             iterations = 1003  # number of pbkdf2 iterations on mac
             self.key = PBKDF2(my_pass, self.salt,
                               iterations=iterations).read(self.length)
-            
+
             cookie_file = self.cookie_file or expand_paths(osx_cookies,'osx')
-        
+
         elif sys.platform.startswith('linux'):
             my_pass = get_linux_pass(os_crypt_name)
 
             iterations = 1
             self.key = PBKDF2(my_pass, self.salt,
                               iterations=iterations).read(self.length)
-            
+
             cookie_file = self.cookie_file or expand_paths(linux_cookies, 'linux')
 
-        
         elif sys.platform == "win32":
             key_file = self.key_file or expand_paths(windows_keys,'windows')
 
@@ -231,66 +233,63 @@ class ChromiumBased:
                     _, self.key = crypt_unprotect_data(keydpapi, is_key=True)
 
             # get cookie file from APPDATA
-            
+
             cookie_file = self.cookie_file
-            
+
             if not cookie_file:
                 if self.browser.lower() == 'chrome' and windows_group_policy_path():
                     cookie_file = windows_group_policy_path()
                 else:
                     cookie_file = expand_paths(windows_cookies,'windows')
-        
+
         else:
             raise BrowserCookieError(
                 "OS not recognized. Works on OSX, Windows, and Linux.")
-        
+
         if not cookie_file:
                 raise BrowserCookieError('Failed to find {} cookie'.format(self.browser))
-        
+
         self.tmp_cookie_file = create_local_copy(cookie_file)
 
     def __del__(self):
         # remove temporary backup of sqlite cookie database
         if hasattr(self, 'tmp_cookie_file'):  # if there was an error till here
             os.remove(self.tmp_cookie_file)
-    
+
     def __str__(self):
         return self.browser
-    
+
     def load(self):
-        """Load sqlite cookies into a cookiejar
-        """
+        """Load sqlite cookies into a cookiejar"""
         con = sqlite3.connect(self.tmp_cookie_file)
+        con.text_factory = text_factory
         cur = con.cursor()
         try:
             # chrome <=55
-            cur.execute('SELECT host_key, path, secure, expires_utc, name, value, encrypted_value '
-                        'FROM cookies WHERE host_key like "%{}%";'.format(self.domain_name))
+            cur.execute('SELECT host_key, path, secure, expires_utc, name, value, encrypted_value, is_httponly '
+                        'FROM cookies WHERE host_key like ?;', ('%{}%'.format(self.domain_name),))
         except sqlite3.OperationalError:
             # chrome >=56
-            cur.execute('SELECT host_key, path, is_secure, expires_utc, name, value, encrypted_value '
-                        'FROM cookies WHERE host_key like "%{}%";'.format(self.domain_name))
+            cur.execute('SELECT host_key, path, is_secure, expires_utc, name, value, encrypted_value, is_httponly '
+                        'FROM cookies WHERE host_key like ?;', ('%{}%'.format(self.domain_name),))
 
         cj = http.cookiejar.CookieJar()
-        epoch_start = datetime.datetime(1601, 1, 1)
-        for item in cur.fetchall():
-            host, path, secure, expires, name = item[:5]
-            if item[3] != 0:
-                # ensure dates don't exceed the datetime limit of year 10000
-                try:
-                    offset = min(int(item[3]), 265000000000000000)
-                    delta = datetime.timedelta(microseconds=offset)
-                    expires = epoch_start + delta
-                    expires = expires.timestamp()
-                # Windows 7 has a further constraint
-                except OSError:
-                    offset = min(int(item[3]), 32536799999000000)
-                    delta = datetime.timedelta(microseconds=offset)
-                    expires = epoch_start + delta
-                    expires = expires.timestamp()
 
-            value = self._decrypt(item[5], item[6])
-            c = create_cookie(host, path, secure, expires, name, value)
+        for item in cur.fetchall():
+            # Per https://github.com/chromium/chromium/blob/main/base/time/time.h#L5-L7,
+            # Chromium-based browsers store cookies' expiration timestamps as MICROSECONDS elapsed
+            # since the Windows NT epoch (1601-01-01 0:00:00 GMT), or 0 for session cookies.
+            #
+            # http.cookiejar stores cookies' expiration timestamps as SECONDS since the Unix epoch
+            # (1970-01-01 0:00:00 GMT, or None for session cookies.
+            host, path, secure, expires_nt_time_epoch, name, value, enc_value, http_only = item
+            if (expires_nt_time_epoch == 0):
+                expires = None
+            else:
+                expires = (expires_nt_time_epoch / 1000000) - self.UNIX_TO_NT_EPOCH_OFFSET
+
+            value = self._decrypt(value, enc_value)
+            c = create_cookie(host, path, secure, expires, name, value, http_only)
             cj.set_cookie(c)
         con.close()
         return cj
@@ -309,8 +308,7 @@ class ChromiumBased:
         return data.decode()
 
     def _decrypt(self, value, encrypted_value):
-        """Decrypt encoded cookies
-        """
+        """Decrypt encoded cookies"""
 
         if sys.platform == 'win32':
             try:
@@ -345,7 +343,7 @@ class ChromiumBased:
 
         cipher = pyaes.Decrypter(
             pyaes.AESModeOfOperationCBC(self.key, self.iv))
-        
+
         # will rise Value Error: invalid padding byte if the key is wrong,
         # probably we did not got the key and used peanuts
         try:
@@ -357,6 +355,7 @@ class ChromiumBased:
         return decrypted.decode("utf-8")
 
 class Chrome(ChromiumBased):
+    """Class for Google Chrome"""
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies':[
@@ -366,7 +365,10 @@ class Chrome(ChromiumBased):
             'windows_cookies':[
                     {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Cookies'},
                     {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'}
+                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'}
                 ],
             'osx_cookies': ['~/Library/Application Support/Google/Chrome/Default/Cookies'],
             'windows_keys': [
@@ -382,13 +384,17 @@ class Chrome(ChromiumBased):
         super().__init__(browser='Chrome', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 class Chromium(ChromiumBased):
+    """Class for Chromium"""
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies':['~/.config/chromium/Default/Cookies'],
             'windows_cookies':[
                     {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Cookies'},
                     {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'}
+                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'}
             ],
             'osx_cookies': ['~/Library/Application Support/Chromium/Default/Cookies'],
             'windows_keys': [
@@ -403,13 +409,17 @@ class Chromium(ChromiumBased):
         super().__init__(browser='Chromium', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 class Opera(ChromiumBased):
+    """Class for Opera"""
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies': ['~/.config/opera/Cookies'],
             'windows_cookies':[
                     {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Cookies'},
                     {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
-                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'}
+                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Network\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'},
+                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'}
             ],
             'osx_cookies': ['~/Library/Application Support/com.operasoftware.Opera/Cookies'],
             'windows_keys': [
@@ -424,7 +434,43 @@ class Opera(ChromiumBased):
 
         super().__init__(browser='Opera', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
+class Brave(ChromiumBased):
+    def __init__(self, cookie_file=None, domain_name="", key_file=None):
+        args = {
+            'linux_cookies':[
+                    '~/.config/BraveSoftware/Brave-Browser/Default/Cookies',
+                    '~/.config/BraveSoftware/Brave-Browser-Beta/Default/Cookies'
+            ],
+            'windows_cookies':[
+                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+            ],
+            'osx_cookies': [
+                    '~/Library/Application Support/BraveSoftware/Brave-Browser/Default/Cookies',
+                    '~/Library/Application Support/BraveSoftware/Brave-Browser-Beta/Default/Cookies'
+            ],
+            'windows_keys': [
+                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'},
+                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browse-Betar\\User Data\\Local State'},
+                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'}
+            ],
+            'os_crypt_name':'brave',
+            'osx_key_service' : 'Brave Safe Storage',
+            'osx_key_user' : 'Brave'
+        }
+        super().__init__(browser='Brave', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 class Edge(ChromiumBased):
+    """Class for Microsoft Edge"""
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies': [
@@ -434,7 +480,10 @@ class Edge(ChromiumBased):
             'windows_cookies':[
                     {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Cookies'},
                     {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'}
+                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
+                    {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
+                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'}
             ],
             'osx_cookies': ['~/Library/Application Support/Microsoft Edge/Default/Cookies'],
             'windows_keys': [
@@ -450,6 +499,7 @@ class Edge(ChromiumBased):
         super().__init__(browser='Edge', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 class Firefox:
+    """Class for Firefox"""
     def __init__(self, cookie_file=None, domain_name=""):
         self.tmp_cookie_file = None
         cookie_file = cookie_file or self.find_cookie_file()
@@ -481,7 +531,7 @@ class Firefox:
             return fallback_path
 
         profiles_ini_path = profiles_ini_path[0]
-        config.read(profiles_ini_path)
+        config.read(profiles_ini_path, encoding="utf8")
 
         profile_path = None
         for section in config.sections():
@@ -525,13 +575,14 @@ class Firefox:
         if cookie_files:
             return cookie_files[0]
         else:
-            raise BrowserCookieError('Failed to find Firefox cookie')
+            raise BrowserCookieError('Failed to find Firefox cookie file')
 
     @staticmethod
     def __create_session_cookie(cookie_json):
-        expires = str(int(time.time()) + 3600 * 24 * 7)
-        return create_cookie(cookie_json.get('host', ''), cookie_json.get('path', ''), False, expires,
-                             cookie_json.get('name', ''), cookie_json.get('value', ''))
+        return create_cookie(cookie_json.get('host', ''), cookie_json.get('path', ''),
+                             cookie_json.get('secure', False), None,
+                             cookie_json.get('name', ''), cookie_json.get('value', ''),
+                             cookie_json.get('httponly', False))
 
     def __add_session_cookies(self, cj):
         if not os.path.exists(self.session_file):
@@ -564,12 +615,13 @@ class Firefox:
     def load(self):
         con = sqlite3.connect(self.tmp_cookie_file)
         cur = con.cursor()
-        cur.execute('select host, path, isSecure, expiry, name, value from moz_cookies '
-                    'where host like "%{}%"'.format(self.domain_name))
+        cur.execute('select host, path, isSecure, expiry, name, value, isHttpOnly from moz_cookies '
+                    'where host like ?', ('%{}%'.format(self.domain_name),))
 
         cj = http.cookiejar.CookieJar()
         for item in cur.fetchall():
-            c = create_cookie(*item)
+            host, path, secure, expires, name, value, http_only = item
+            c = create_cookie(host, path, secure, expires, name, value, http_only)
             cj.set_cookie(c)
         con.close()
 
@@ -579,11 +631,12 @@ class Firefox:
         return cj
 
 
-def create_cookie(host, path, secure, expires, name, value):
-    """Shortcut function to create a cookie
-    """
+def create_cookie(host, path, secure, expires, name, value, http_only):
+    """Shortcut function to create a cookie"""
+    # HTTPOnly flag goes in _rest, if present (see https://github.com/python/cpython/pull/17471/files#r511187060)
     return http.cookiejar.Cookie(0, name, value, None, False, host, host.startswith('.'), host.startswith('.'), path,
-                                 True, secure, expires, False, None, None, {})
+                                 True, secure, expires, False, None, None,
+                                 {'HTTPOnly': ''} if http_only else {})
 
 def chrome(cookie_file=None, domain_name="", key_file=None):
     """Returns a cookiejar of the cookies used by Chrome. Optionally pass in a
@@ -603,6 +656,12 @@ def opera(cookie_file=None, domain_name="", key_file=None):
     """
     return Opera(cookie_file, domain_name, key_file).load()
 
+def brave(cookie_file=None, domain_name="", key_file=None):
+    """Returns a cookiejar of the cookies and sessions used by Brave. Optionally
+    pass in a domain name to only load cookies from the specified domain
+    """
+    return Brave(cookie_file, domain_name, key_file).load()
+
 def edge(cookie_file=None, domain_name="", key_file=None):
     """Returns a cookiejar of the cookies used by Microsoft Egde. Optionally pass in a
     domain name to only load cookies from the specified domain
@@ -621,7 +680,7 @@ def load(domain_name=""):
     Optionally pass in a domain name to only load cookies from the specified domain
     """
     cj = http.cookiejar.CookieJar()
-    for cookie_fn in [chrome, chromium, opera, edge, firefox]:
+    for cookie_fn in [chrome, chromium, opera, brave, edge, firefox]:
         try:
             for cookie in cookie_fn(domain_name=domain_name):
                 cj.set_cookie(cookie)
